# interactive console set date and time
# Paul Mirel 2023
# NASA open source software license

## look up UTC time here: https://time.is/UTC

import busio
from adafruit_pcf8523 import pcf8523
import time
#import rtc    # import the microcontroller system real time clock functionality
import board
import displayio
import terminalio
from adafruit_display_text import label
try:
    import adafruit_ssd1327
except:
    display = False

displayio.release_displays()

try:
    i2c_bus = board.I2C()
    print( "i2c bus initialized" )
except:
    print( "i2c bus failed to initialize" )
    i2c_bus = False


try:
    real_time_clock = pcf8523.PCF8523(i2c_bus)
    print( "real time clock initialized" )
except:
    print( "real time clock failed to initialize" )
    real_time_clock = False

days = ("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday")
# wday =    0         1         2           3            4          5          6
if False: #True:    # if False, do not change the time setting on the real time clock module.
                    # if True, write the new time setting from below onto the real time clock module.

    #                   (( year, mon, date, hour, min, sec, wday, yday, isdst ))
    t = time.struct_time(( 2022,  12,   30,   22,  21,  15,   5,   -1,    -1 ))
    # you must set year, mon, date, hour, min, sec and weekday
    # yday = day of the year is not supported by this clock,
    # isdst = 'is daylight savings time' can be set but we don't do anything with it at this time

    print("Setting time to:", t)     # uncomment for debugging
    real_time_clock.datetime = t
    print()
if real_time_clock:
    battery_low = real_time_clock.battery_low
else:
    battery_low = False

if battery_low:
    print( "Clock battery is low. Replace the clock battery." )
else:
    print( "Clock battery voltage is OK." )

try:
    display_bus = displayio.I2CDisplay(i2c_bus, device_address=0x3D)
    display = adafruit_ssd1327.SSD1327(display_bus, width=128, height=128)
    print( "display initialized" )
    display = True
except:
    print( "display failed to initialize" )
    display = False

if display:
        display_group = displayio.Group()
        display.show( display_group )
        background_bitmap = displayio.Bitmap( 128, 128, 1 )
        background_palette = displayio.Palette(1)
        background_palette[0] = 0x888888  # Gray
        background_square = displayio.TileGrid( background_bitmap, pixel_shader=background_palette, x=0, y=0 )
        display_group.append(background_square)

        text = "Year"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=2, x=5, y=9 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "Month"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=2, x=5, y=29 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "Day"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=2, x=5, y=49 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "HourUTC"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=2, x=5, y=69 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "Min"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=2, x=5, y=89 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "Sec"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=2, x=5, y=109 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "--"
        yy_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        yy_text_group = displayio.Group( scale=2, x=75, y=9 )
        yy_text_group.append( yy_text_area )
        display_group.append( yy_text_group )

        text = "--"
        mo_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        mo_text_group = displayio.Group( scale=2, x=95, y=29 )
        mo_text_group.append( mo_text_area )
        display_group.append( mo_text_group )

        text = "--"
        dd_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        dd_text_group = displayio.Group( scale=2, x=95, y=49 )
        dd_text_group.append( dd_text_area )
        display_group.append( dd_text_group )

        text = "---"
        wd_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        wd_text_group = displayio.Group( scale=2, x=50, y=49 )
        wd_text_group.append( wd_text_area )
        display_group.append( wd_text_group )

        text = "--"
        hh_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        hh_text_group = displayio.Group( scale=2, x=95, y=69 )
        hh_text_group.append( hh_text_area )
        display_group.append( hh_text_group )

        text = "--"
        mi_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        mi_text_group = displayio.Group( scale=2, x=95, y=89 )
        mi_text_group.append( mi_text_area )
        display_group.append( mi_text_group )

        text = "--"
        ss_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        ss_text_group = displayio.Group( scale=2, x=95, y=109 )
        ss_text_group.append( ss_text_area )
        display_group.append( ss_text_group )

        if battery_low:
            text = "clock bat LOW"
        else:
            text = "clock bat OK"
        cb_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        cb_text_group = displayio.Group( scale=1, x=48, y=123 )
        cb_text_group.append( cb_text_area )
        display_group.append( cb_text_group )




try:
    while True:
        timenow = real_time_clock.datetime
        #print(timenow)  # hardware clock time
        #feather_local_time = time.localtime()
        #print(feather_local_time)       # microcontroller local time
        print()
        weekday = timenow.tm_wday
        year = timenow.tm_year
        month = timenow.tm_mon
        day = timenow.tm_mday
        hour = timenow.tm_hour
        minute = timenow.tm_min
        second = timenow.tm_sec
        previous_weekday = weekday
        previous_year = year
        previous_month = month
        previous_day = day
        previous_hour = hour
        previous_minute = minute
        previous_second = second
        if display:
            yy_text_area.text = str(year)
            mo_text_area.text = str(month)
            dd_text_area.text = str(day)
            wd_text_area.text = days[ weekday ][0:3]
            hh_text_area.text = str(hour)
            mi_text_area.text = str(minute)
            ss_text_area.text = str(second)

        try:
            print( "The date is %s %d-%d-%d" % ( days[ weekday ], year, month, day ))
            print( "The time is %d:%02d:%02d" % ( hour, minute, second ))
        except IndexError as err:
            print( "The clock has not been set, and the values are out of range." )

        print()
        print( "Current year is {}. Enter a new year and press return, or press return to skip.".format(timenow.tm_year))
        print(">", end = ' ')
        input_string = False
        while input_string == False:
            input_string = input().strip()
        try:
            input_integer = int( input_string )
        except ValueError:
            input_integer = 2000
        if input_integer in range (2010, 2100):
            year = input_integer
            t = time.struct_time(( year,  month,   day,   hour,  minute,  second,   weekday,   -1,    -1 ))
            real_time_clock.datetime = t

        print()
        print( "Current month is {}. Enter a new month and press return, or press return to skip.".format(timenow.tm_mon))
        print(">", end = ' ')
        input_string = False
        while input_string == False:
            input_string = input().strip()
        try:
            input_integer = int( input_string )
        except ValueError:
            pass
        if input_integer in range (1, 12):
            month = input_integer
            t = time.struct_time(( year,  month,   day,   hour,  minute,  second,   weekday,   -1,    -1 ))
            real_time_clock.datetime = t

        print()
        print( "Current day is {}. Enter a new day and press return, or press return to skip.".format(timenow.tm_mday))
        print(">", end = ' ')
        input_string = False
        while input_string == False:
            input_string = input().strip()
        try:
            input_integer = int( input_string )
        except ValueError:
            pass
        if input_integer in range (1, 32):
            day = input_integer
            t = time.struct_time(( year,  month,   day,   hour,  minute,  second,   weekday,   -1,    -1 ))
            real_time_clock.datetime = t

        print()
        print( "Current hour is {} UTC. Enter a new hour and press return, or press return to skip.".format(timenow.tm_hour))
        print(">", end = ' ')
        input_string = False
        while input_string == False:
            input_string = input().strip()
        try:
            input_integer = int( input_string )
        except ValueError:
            pass
        if input_integer in range (0, 23):
            hour = input_integer
            t = time.struct_time(( year,  month,   day,   hour,  minute,  second,   weekday,   -1,    -1 ))
            real_time_clock.datetime = t

        print()
        print( "Current minute is {}. Enter a new minute and press return, or press return to skip.".format(timenow.tm_min))
        print(">", end = ' ')
        input_string = False
        while input_string == False:
            input_string = input().strip()
        try:
            input_integer = int( input_string )
        except ValueError:
            pass
        if input_integer in range (0, 59):
            minute = input_integer
            t = time.struct_time(( year,  month,   day,   hour,  minute,  second,   weekday,   -1,    -1 ))
            real_time_clock.datetime = t

        print()
        print( "Current second is {}. Enter a new second and press return, or press return to skip.".format(timenow.tm_sec))
        print(">", end = ' ')
        input_string = False
        while input_string == False:
            input_string = input().strip()
        try:
            input_integer = int( input_string )
        except ValueError:
            pass
        if input_integer in range (0, 59):
            second = input_integer
            t = time.struct_time(( year,  month,   day,   hour,  minute,  second,   weekday,   -1,    -1 ))
            real_time_clock.datetime = t

        print()
        print( "Current weekday is {}. Enter a new weekday and press return, or press return to skip.".format(days[timenow.tm_wday]))
        print( "Enter: sun, mon, tue, wed, thu, fri, sat" )
        print(">", end = ' ')
        input_string = False

        input_string = input().strip()
        print(input_string)
        if input_string == "mon":
            weekday = 1
        elif input_string == "tue":
            weekday = 2
        elif input_string == "wed":
            weekday = 3
        elif input_string == "thu":
            weekday = 4
        elif input_string == "fri":
            weekday = 5
        elif input_string == "sat":
            weekday = 6
        elif input_string == "sun":
            weekday = 0
        else:
            pass
        print("weekday = {}".format(weekday))
        t = time.struct_time(( year,  month,   day,   hour,  minute,  second,   weekday,   -1,    -1 ))
        real_time_clock.datetime = t

        time.sleep( 0.1 )
finally:
        displayio.release_displays()
        print( "\ndisplayio displays released" )
        i2c_bus.deinit()
        print( "i2c_bus deinitialized" )
        print( " ---- end --- " )
