import time
import board
import busio
import displayio
import terminalio
from adafruit_display_text import label
import adafruit_ssd1327
from digitalio import DigitalInOut, Direction, Pull
from adafruit_pm25.i2c import PM25_I2C

def main():
    reset_pin = None

    try:
        i2c_bus = busio.I2C(board.SCL, board.SDA, frequency=100000)
        # slow the bus down from 400 to 100 kHz: particle sensor works better at the slower speed
        print( "i2c bus initialized" )
    except:
        print( "i2c bus failed to initialize" )
        i2c_bus = False

    try:
        display_bus = displayio.I2CDisplay(i2c_bus, device_address=0x3D)
        display = adafruit_ssd1327.SSD1327(display_bus, width=128, height=128)
        print( "display initialized" )
    except:
        print( "display failed to initialize" )
        display = False

    loop_count = 0

    if display:
        display_group = displayio.Group()
        display.show( display_group )
        background_bitmap = displayio.Bitmap( 128, 128, 1 )
        background_palette = displayio.Palette(1)
        background_palette[0] = 0x444444  # Gray
        background_square = displayio.TileGrid( background_bitmap, pixel_shader=background_palette, x=0, y=0 )
        display_group.append(background_square)

        text = "Particulates Sensor\n      PM2.5"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=1, x=8, y=15 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "size um: count/100mL"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=1, x=6, y=45 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "0.3 -- "
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=1, x=20, y=59 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "0.5 -- "
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=1, x=20, y=70 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "1.0 -- "
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=1, x=20, y=81 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "2.5 -- "
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=1, x=20, y=92 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "5.0 -- "
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=1, x=20, y=103 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "10. -- "
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=1, x=20, y=114 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "--"
        u03_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        u03_text_group = displayio.Group( scale=1, x=60, y=59 )
        u03_text_group.append( u03_text_area )
        display_group.append( u03_text_group )

        text = "--"
        u05_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        u05_text_group = displayio.Group( scale=1, x=60, y=70 )
        u05_text_group.append( u05_text_area )
        display_group.append( u05_text_group )

        text = "--"
        u1_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        u1_text_group = displayio.Group( scale=1, x=60, y=81 )
        u1_text_group.append( u1_text_area )
        display_group.append( u1_text_group )

        text = "--"
        u2_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        u2_text_group = displayio.Group( scale=1, x=60, y=92 )
        u2_text_group.append( u2_text_area )
        display_group.append( u2_text_group )

        text = "--"
        u5_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        u5_text_group = displayio.Group( scale=1, x=60, y=103 )
        u5_text_group.append( u5_text_area )
        display_group.append( u5_text_group )

        text = "--"
        u10_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        u10_text_group = displayio.Group( scale=1, x=60, y=114 )
        u10_text_group.append( u10_text_area )
        display_group.append( u10_text_group )

        text = "AQIp"
        aqip_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        aqip_text_group = displayio.Group( scale=1, x=95, y=81 )
        aqip_text_group.append( aqip_text_area )
        display_group.append( aqip_text_group )

        text = "--"
        aqipd_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        aqipd_text_group = displayio.Group( scale=1, x=95, y=92 )
        aqipd_text_group.append( aqipd_text_area )
        display_group.append( aqipd_text_group )


    try:
        pm25 = PM25_I2C(i2c_bus, reset_pin)
    except:
        pm25 = False

    print("Found PM2.5 sensor, reading data...")

    try:
        while True:
            time.sleep(1)

            try:
                aqdata = pm25.read()
                # print(aqdata)
            except RuntimeError:
                print("Unable to read from sensor, retrying...")
                continue

            print()
            print("Concentration Units (standard)")
            print("---------------------------------------")
            print(
                "PM 1.0: %d\tPM2.5: %d\tPM10: %d"
                % (aqdata["pm10 standard"], aqdata["pm25 standard"], aqdata["pm100 standard"])
            )
            print("Concentration Units (environmental)")
            print("---------------------------------------")
            print(
                "PM 1.0: %d\tPM2.5: %d\tPM10: %d"
                % (aqdata["pm10 env"], aqdata["pm25 env"], aqdata["pm100 env"])
            )
            print("---------------------------------------")
            print("Particles > 0.3um / 0.1L air:", aqdata["particles 03um"])
            print("Particles > 0.5um / 0.1L air:", aqdata["particles 05um"])
            print("Particles > 1.0um / 0.1L air:", aqdata["particles 10um"])
            print("Particles > 2.5um / 0.1L air:", aqdata["particles 25um"])
            print("Particles > 5.0um / 0.1L air:", aqdata["particles 50um"])
            print("Particles > 10 um / 0.1L air:", aqdata["particles 100um"])
            if display:
                u03_text_area.text = str(aqdata["particles 03um"])
                u05_text_area.text = str(aqdata["particles 05um"])
                u1_text_area.text = str(aqdata["particles 10um"])
                u2_text_area.text = str(aqdata["particles 25um"])
                u5_text_area.text = str(aqdata["particles 50um"])
                u10_text_area.text = str(aqdata["particles 100um"])
            print("---------------------------------------")
            p25_reading = aqdata["particles 25um"]
            p100_reading = aqdata["particles 100um"]
            aqi = ( calculate_aqi ( p25_reading, p100_reading ))
            print("AQI = {}".format(aqi))
            aqipd_text_area.text = str(aqi)

    finally:
            displayio.release_displays()
            print( "displayio displays released" )
            if i2c_bus:
                i2c_bus.deinit()
                print( "i2c_bus deinitialized" )
            print( " ---- end --- " )

def calculate_aqi( p25_reading, p100_reading ):
    p25_break_points = (0, 12, 12.1, 35.4, 35.5, 55.4, 55.5, 150.4, 150.5, 250.4, 250.5, 350.4, 350.5, 500.4)
    #print( p25_break_points )
    p100_break_points = (0, 54, 55, 154, 155, 254, 255, 354, 355, 424, 425, 504, 505, 604)
    #print( p100_break_points )
    AQI_levels = (0, 50, 51, 100, 101, 150, 151, 200, 201, 300, 301, 400, 401, 500)
    #print( AQI_levels )
    p25_index = 0
    p100_index = 0

    for i in range (len(AQI_levels)-1):
        #print(i)
        if (p25_reading > p25_break_points[i] and p25_reading < p25_break_points[i+1]):
            p25_index = i
            #print(i)

    #aqi = int((((ihi-ilo)/(bhi-blo))*(reading-blo))+ilo)

    aqi25 = int((((AQI_levels[p25_index+1] - AQI_levels[p25_index])/(p25_break_points[p25_index+1] - p25_break_points[p25_index]))*(p25_reading-p25_break_points[p25_index]))+AQI_levels[p25_index])
    #print(aqi25)

    for n in range (len(AQI_levels)-1):
        #print(i)
        if (p100_reading > p100_break_points[i] and p100_reading < p100_break_points[i+1]):
            p100_index = i
            #print(i)

    aqi100 = int((((AQI_levels[p100_index+1] - AQI_levels[p100_index])/(p100_break_points[p100_index+1] - p100_break_points[p100_index]))*(p100_reading-p25_break_points[p100_index]))+AQI_levels[p100_index])
    #print(aqi100)

    #print( p25_break_points[p25_index], p25_break_points[p25_index+1], AQI_levels[p25_index], AQI_levels[p25_index+1], p100_break_points[p100_index], p100_break_points[p100_index+1], AQI_levels[p100_index], AQI_levels[p100_index+1])

    if aqi100 > aqi25:
        return aqi100
    else:
        return aqi25

main()
