import time
import board
from adafruit_bme280 import basic as adafruit_bme280
import displayio
import terminalio
from adafruit_display_text import label
import adafruit_ssd1327

displayio.release_displays()

try:
    i2c_bus = board.I2C()
    print( "i2c bus initialized" )
except:
    print( "i2c bus failed to initialize" )
    i2c_bus = False

try:
    display_bus = displayio.I2CDisplay(i2c_bus, device_address=0x3D)
    display = adafruit_ssd1327.SSD1327(display_bus, width=128, height=128)
    print( "display initialized" )
except:
    print( "display failed to initialize" )
    display = False

if display:
        display_group = displayio.Group()
        display.show( display_group )
        background_bitmap = displayio.Bitmap( 128, 128, 1 )
        background_palette = displayio.Palette(1)
        background_palette[0] = 0x888888  # Gray
        background_square = displayio.TileGrid( background_bitmap, pixel_shader=background_palette, x=0, y=0 )
        display_group.append(background_square)

        text = "Weather"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=2, x=25, y=15 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "Temperature"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=1, x=8, y=45 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "Pressure"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=1, x=8, y=70 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "Humidity"
        text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        text_group = displayio.Group( scale=1, x=8, y=95 )
        text_group.append( text_area )
        display_group.append( text_group )

        text = "--"
        temp_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        temp_text_group = displayio.Group( scale=1, x=85, y=45 )
        temp_text_group.append( temp_text_area )
        display_group.append( temp_text_group )

        text = "--"
        press_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        press_text_group = displayio.Group( scale=1, x=65, y=70 )
        press_text_group.append( press_text_area )
        display_group.append( press_text_group )

        text = "--"
        hum_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
        hum_text_group = displayio.Group( scale=1, x=70, y=95 )
        hum_text_group.append( hum_text_area )
        display_group.append( hum_text_group )

try:
    bme280 = adafruit_bme280.Adafruit_BME280_I2C( i2c_bus )
    print( "weather sensor initialized" )
except:
    bme280 = False
    print( "weather sensor failed to initialize" )


if bme280:
    # change this to match the location's pressure (hPa) at sea level
    bme280.sea_level_pressure = 1013.25
try:
    while True:
        if bme280:
            print("\nTemperature: %0.1f C" % bme280.temperature)
            print("Humidity: %0.1f %%" % bme280.relative_humidity)
            print("Pressure: %0.1f hPa" % bme280.pressure)
            print("Altitude = %0.2f meters" % bme280.altitude)
            if display:
                temp_text_area.text = "{:.1f} C".format( bme280.temperature )
                hum_text_area.text = "{:.1f} %".format( bme280.relative_humidity)
                press_text_area.text = "{:.1f} hPa".format( bme280.pressure)

        time.sleep(1)

finally:
        displayio.release_displays()
        print( "displayio displays released" )
        i2c_bus.deinit()
        print( "i2c_bus deinitialized" )
        print( " ---- end --- " )
