import board
import time
try:
    import adafruit_max1704x
except:
    print( "max1704x library not available" )
from analogio import AnalogIn

try:
    vbat_voltage_pin = AnalogIn(board.VOLTAGE_MONITOR)
except AttributeError:
    print( "analog voltage monitor not available" )
    vbat_voltage_pin = False

try:
    monitor = adafruit_max1704x.MAX17048(board.I2C())
except:
    print( "fuel gauge battery monitor not available" )
    monitor = False

def get_voltage(pin):
    if pin:
        return (pin.value * 3.3) / 65536 * 2
    else:
        return 0

if vbat_voltage_pin:
    print( "getting analog voltage reading" )
    battery_voltage = get_voltage(vbat_voltage_pin)
    battery_percentage = 0

if monitor:
    print( "getting fuel gauge battery health" )
    battery_voltage = monitor.cell_voltage          #wake up the fuel gauge chip with a couple of throw away commands
    battery_percentage = monitor.cell_percent
    time.sleep(0.1)
    battery_voltage = monitor.cell_voltage
    battery_percentage = monitor.cell_percent

print("Main battery voltage: {:.2f} V".format(battery_voltage))
print("Main battery percentage: {:.0f} %".format(battery_percentage))
